@@ -35,7 +35,6 @@
 #include <string.h>
 
 #include "private/common.h"
-#include "private/sse2_64_32.h"
 
 #ifdef HAVE_EMMINTRIN_H
 
@@ -46,21 +45,17 @@
 # if defined(__XOP__) && defined(DISABLED)
 #  include <x86intrin.h>
 # endif
+# include "private/sse2_64_32.h"
 
 # include "../crypto_scrypt.h"
 # include "../pbkdf2-sha256.h"
 
-# if defined(__XOP__) && defined(DISABLED)
-#  define ARX(out, in1, in2, s) \
-    out = _mm_xor_si128(out, _mm_roti_epi32(_mm_add_epi32(in1, in2), s));
-# else
-#  define ARX(out, in1, in2, s)                                    \
+# define ARX(out, in1, in2, s)                                     \
     {                                                              \
         __m128i T = _mm_add_epi32(in1, in2);                       \
         out       = _mm_xor_si128(out, _mm_slli_epi32(T, s));      \
         out       = _mm_xor_si128(out, _mm_srli_epi32(T, 32 - s)); \
     }
-# endif
 
 # define SALSA20_2ROUNDS              \
     /* Operate on "columns". */       \
@@ -85,7 +80,7 @@
     X2 = _mm_shuffle_epi32(X2, 0x4E); \
     X3 = _mm_shuffle_epi32(X3, 0x93);
 
-/**
+/*
  * Apply the salsa20/8 core to the block provided in (X0 ... X3) ^ (Z0 ... Z3).
  */
 # define SALSA20_8_XOR(in, out)                               \
@@ -103,10 +98,11 @@
         (out)[3] = X3 = _mm_add_epi32(X3, Y3);                \
     }
 
-/**
+/*
  * blockmix_salsa8(Bin, Bout, r):
- * Compute Bout = BlockMix_{salsa20/8, r}(Bin).  The input Bin must be 128r
- * bytes in length; the output Bout must also be the same size.
+ * Compute Bout = BlockMix_{salsa20/8, r}(Bin).
+ * The input Bin must be 128r bytes in length;
+ * the output Bout must also be the same size.
  */
 static inline void
 blockmix_salsa8(const __m128i *Bin, __m128i *Bout, size_t r)
@@ -208,18 +204,22 @@ blockmix_salsa8_xor(const __m128i *Bin1, const __m128i *Bin2, __m128i *Bout,
 # undef XOR4
 # undef XOR4_2
 
-/**
+/*
  * integerify(B, r):
  * Return the result of parsing B_{2r-1} as a little-endian integer.
  * Note that B's layout is permuted compared to the generic implementation.
  */
-static inline uint32_t
-integerify(const void *B, size_t r)
+static inline uint64_t
+integerify(const __m128i *B, size_t r)
 {
-    return *(const uint32_t *) ((uintptr_t)(B) + (2 * r - 1) * 64);
+    const __m128i * X  = B + (2*r - 1) * 4;
+    const uint32_t X0  = (uint32_t) _mm_cvtsi128_si32(X[0]);
+    const uint32_t X13 = (uint32_t) _mm_cvtsi128_si32(_mm_srli_si128(X[3], 4));
+
+    return (((uint64_t)(X13) << 32) + X0);
 }
 
-/**
+/*
  * smix(B, r, N, V, XY):
  * Compute B = SMix_r(B, N).  The input B must be 128r bytes in length;
  * the temporary storage V must be 128rN bytes in length; the temporary
@@ -228,12 +228,12 @@ integerify(const void *B, size_t r)
  * multiple of 64 bytes.
  */
 static void
-smix(uint8_t *B, size_t r, uint32_t N, void *V, void *XY)
+smix(uint8_t *B, size_t r, uint64_t N, void *V, void *XY)
 {
     size_t    s   = 128 * r;
-    __m128i * X   = (__m128i *) V, *Y;
+    __m128i  *X   = (__m128i *) V, *Y;
     uint32_t *X32 = (uint32_t *) V;
-    uint32_t  i, j;
+    uint64_t  i, j;
     size_t    k;
 
     /* 1: X <-- B */
@@ -295,7 +295,7 @@ smix(uint8_t *B, size_t r, uint32_t N, void *V, void *XY)
     }
 }
 
-/**
+/*
  * escrypt_kdf(local, passwd, passwdlen, salt, saltlen,
  *     N, r, p, buf, buflen):
  * Compute scrypt(passwd[0 .. passwdlen - 1], salt[0 .. saltlen - 1], N, r,
@@ -318,10 +318,12 @@ escrypt_kdf_sse(escrypt_local_t *local, const uint8_t *passwd, size_t passwdlen,
 
 /* Sanity-check parameters. */
 # if SIZE_MAX > UINT32_MAX
+/* LCOV_EXCL_START */
     if (buflen > (((uint64_t)(1) << 32) - 1) * 32) {
         errno = EFBIG;
         return -1;
     }
+/* LCOV_EXCL_END */
 # endif
     if ((uint64_t)(r) * (uint64_t)(p) >= ((uint64_t) 1 << 30)) {
         errno = EFBIG;
@@ -339,6 +341,7 @@ escrypt_kdf_sse(escrypt_local_t *local, const uint8_t *passwd, size_t passwdlen,
         errno = EINVAL;
         return -1;
     }
+/* LCOV_EXCL_START */
     if ((r > SIZE_MAX / 128 / p) ||
 # if SIZE_MAX / 256 <= UINT32_MAX
         (r > SIZE_MAX / 256) ||
@@ -347,26 +350,31 @@ escrypt_kdf_sse(escrypt_local_t *local, const uint8_t *passwd, size_t passwdlen,
         errno = ENOMEM;
         return -1;
     }
+/* LCOV_EXCL_END */
 
     /* Allocate memory. */
     B_size = (size_t) 128 * r * p;
     V_size = (size_t) 128 * r * N;
     need   = B_size + V_size;
+/* LCOV_EXCL_START */
     if (need < V_size) {
         errno = ENOMEM;
         return -1;
     }
+/* LCOV_EXCL_END */
     XY_size = (size_t) 256 * r + 64;
     need += XY_size;
+/* LCOV_EXCL_START */
     if (need < XY_size) {
         errno = ENOMEM;
         return -1;
     }
+/* LCOV_EXCL_END */
     if (local->size < need) {
-        if (free_region(local)) {
+        if (escrypt_free_region(local)) {
             return -1; /* LCOV_EXCL_LINE */
         }
-        if (!alloc_region(local, need)) {
+        if (!escrypt_alloc_region(local, need)) {
             return -1; /* LCOV_EXCL_LINE */
         }
     }
@@ -375,16 +383,16 @@ escrypt_kdf_sse(escrypt_local_t *local, const uint8_t *passwd, size_t passwdlen,
     XY = (uint32_t *) ((uint8_t *) V + V_size);
 
     /* 1: (B_0 ... B_{p-1}) <-- PBKDF2(P, S, 1, p * MFLen) */
-    PBKDF2_SHA256(passwd, passwdlen, salt, saltlen, 1, B, B_size);
+    escrypt_PBKDF2_SHA256(passwd, passwdlen, salt, saltlen, 1, B, B_size);
 
     /* 2: for i = 0 to p - 1 do */
     for (i = 0; i < p; i++) {
         /* 3: B_i <-- MF(B_i, N) */
-        smix(&B[(size_t) 128 * i * r], r, (uint32_t) N, V, XY);
+        smix(&B[(size_t) 128 * i * r], r, N, V, XY);
     }
 
     /* 5: DK <-- PBKDF2(P, B, 1, dkLen) */
-    PBKDF2_SHA256(passwd, passwdlen, B, B_size, 1, buf, buflen);
+    escrypt_PBKDF2_SHA256(passwd, passwdlen, B, B_size, 1, buf, buflen);
 
     /* Success! */
     return 0;
